/* $Header: helloWorld.Xpex.c,v 2.1 91/07/11 12:37:27 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/* helloWorld.Xpex.c */

#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "Xpexlib.h"
#include "Xpexutil.h"
#include "window.h"

#define BLACK   0
#define WHITE   1
#define RED     2
#define GREEN   3
#define BLUE    4
#define YELLOW  5
#define CYAN    6
#define MAGENTA 7

#define WC_MIN        -10.0
#define WC_MAX         10.0

#define DEG_TO_RAD(_DEG)	((_DEG) * (M_PI/180.0))

#define SetElementPointerToBeginning(_dpy,_s) \
{ \
	static XpexElementPos elem_pos = { \
		Xpex_Beginning, 0 \
	}; \
	XpexSetElementPointer((_dpy), (_s), &elem_pos); \
}

static Display *dpy;
static Drawable drawable;
static XpexStructure structure1, structure2;
static XpexWksResources wks_res;
static XpexPhigsWks wks1;

XpexViewMap3D map = {
	{ WC_MIN, WC_MAX, WC_MIN, WC_MAX }, /* window limits */
	{ 0.0, 1.0, 0.0, 1.0, 0.0, 1.0 }, /* viewport limits */
	Xpex_PERSPECTIVE, /* projection type */
	{ 0.0, 0.0, 10.0 }, /* projection reference point */
	0.0,  /* view_plane */
	-20.0, /* back_plane */
	10.0 /* front_plane */
};

static XpexCoord3D   vrp = { 0.0, 0.0, 0.0 }; 
static XpexCoord3D   vup = { 0.0, 1.0, 0.0 }; 
/* static XpexCoord3D   vpn = { 0.0, 0.0, 1.0 };  */
static XpexCoord3D   vpn = { 0.0, 0.0, 1.0 }; 

DebugStructure(dpy, sid)
	Display *dpy;
	XpexStructure sid;
{
	XpexStructureInfo info;

	XpexGetStructureInfo(dpy, sid, &info);

	(void) fprintf(stderr, "info.editing_mode: %s\n", 
	info.editing_mode ? "Replace" : "Insert");
	(void) fprintf(stderr, "info.element_ptr: %d\n",
	info.element_ptr);
	(void) fprintf(stderr, "info.num_elements: %d\n",
	info.num_elements);
	(void) fprintf(stderr, "info.total_length: %d\n",
	info.total_length);
	(void) fprintf(stderr, "info.has_refs: %d\n",
	info.has_refs);
}

InitView()
{
	/* Viewing parameters */
	static XpexViewRep  rep;

	if (XpexEvaluateViewMappingMatrix(&map, rep.view.mapping)) {
		(void) fprintf(stderr, "Error from eval mapping\n");
	}

	if (XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup,
	rep.view.orientation)) {
		(void) fprintf(stderr, "Error from eval orientation\n");
	}

	rep.index = 1;
	rep.view.clipFlags = Xpex_ClipXY | Xpex_ClipBack | Xpex_ClipFront;
	/* rep.view.clipFlags = 0x0; */
	rep.view.clipLimits.minval.x = map.viewport.x_min;
	rep.view.clipLimits.minval.y = map.viewport.y_min;
	rep.view.clipLimits.minval.z = map.viewport.z_min;
	rep.view.clipLimits.maxval.x = map.viewport.x_max;
	rep.view.clipLimits.maxval.y = map.viewport.y_max;
	rep.view.clipLimits.maxval.z = map.viewport.z_max;

	XpexSetViewRep(dpy, wks1, &rep);
}

InitWks()
{
	wks_res.line_bundle =
	XpexCreateLineBundleLUT(dpy, drawable);

	wks_res.marker_bundle =
	XpexCreateMarkerBundleLUT(dpy,drawable);

	wks_res.text_bundle =
	XpexCreateTextBundleLUT(dpy,drawable);

	wks_res.interior_bundle =
	XpexCreateInteriorBundleLUT(dpy,drawable);

	wks_res.edge_bundle =
	XpexCreateEdgeBundleLUT(dpy,drawable);

	wks_res.pattern_table =
	XpexCreatePatternLUT(dpy,drawable);

	wks_res.text_font_table =
	XpexCreateTextFontLUT(dpy,drawable);

	wks_res.color_table =
	XpexCreateColorLUT(dpy,drawable);

	wks_res.light_table =
	XpexCreateLightLUT(dpy,drawable);

	wks_res.depth_cue_table =
	XpexCreateDepthCueLUT(dpy,drawable);

	wks_res.color_approx_table =
	XpexCreateColorApproxLUT(dpy,drawable);

	wks_res.highlight_incl = XpexCreateNameSet(dpy);
	wks_res.highlight_excl = XpexCreateNameSet(dpy);
	wks_res.invis_incl = XpexCreateNameSet(dpy);
	wks_res.invis_excl = XpexCreateNameSet(dpy);

	XpexSetupColormap(dpy, drawable, wks_res.color_approx_table);

	wks1 = XpexCreatePhigsWks(dpy,
	drawable, &wks_res, Xpex_SingleBuffered);

	InitView();
}

InitStructures()
{
	static	XpexCoord3D  origin 	= { 0.0, 0.0, 0.0 };
	static	XpexVector3D vector1	= { 1.0, 0.0, 0.0 };
	static	XpexVector3D vector2	= { 0.0, 1.0, 0.0 };
	static	char 		*message	= "Sony PEX Library";

	static XpexMatrix matrix = {
		{ 1.0, 0.0, 0.0, 0.0 },
		{ 0.0, 1.0, 0.0, 0.0 },
		{ 0.0, 0.0, 1.0, 0.0 },
		{ 0.0, 0.0, 0.0, 1.0 }
	};

	static XpexCoord3D view_window[5] = {
		{ WC_MIN, WC_MIN, 0.0 },
		{ WC_MAX, WC_MIN, 0.0 },
		{ WC_MAX, WC_MAX, 0.0 },
		{ WC_MIN, WC_MAX, 0.0 },
		{ WC_MIN, WC_MIN, 0.0 }
	};

	XpexCoord3D points[5];

	structure1 = XpexCreateStructure(dpy);
	structure2 = XpexCreateStructure(dpy);

	XpexStoreOutputCmds(dpy, structure1);
	XpexSetEditingMode(dpy, structure1, XpexStructure_Insert);
	XpexSetEditingMode(dpy, structure2, XpexStructure_Insert);

		XpexSetViewIndex(dpy, 1); 
		XpexLabel(dpy, 1); 
		XpexSetLocalTransform(dpy, Xpex_Replace, matrix); 
		XpexExecuteStructure(dpy, structure2);

	XpexStoreOutputCmds(dpy, structure2);
		XpexSetTextColorIndex(dpy, 7);
		XpexSetTextAlignment(dpy, XpexValign_Half, XpexHalign_Center);
		XpexSetCharHeight(dpy, 1.0);
		XpexText(dpy, &origin, &vector1, &vector2, 
		message, strlen(message));

	DebugStructure(dpy, structure1);
}

InitX()
{
	dpy = XOpenDisplay(NULL);

	if (!dpy) {
		(void) fprintf(stderr, 
		"ERROR: Could not open a connection to X on display %s\n", 
		XDisplayName(NULL));
		exit(1);
	}
}

InitPEX()
{
	if (!XpexInitialize(dpy)) {
		(void) fprintf(stderr,
		"ERROR: PEX not supported on display %s\n",
		XDisplayName(NULL));
		exit(1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

Initialize()
{
	InitX();
	XSynchronize(dpy, 1); 
	InitPEX();
	InitStructures();

	SetWindowLabel("helloWorld");
	SetIconLabel("helloWorld");
	drawable = CreateWindow(dpy);

	InitWks();
}

DrawImage()
{
	register float i;

	static XpexMatrix matrix_a = {
		{ 1.0, 0.0, 0.0, 0.0 },
		{ 0.0, 1.0, 0.0, 0.0 },
		{ 0.0, 0.0, 1.0, 0.0 },
		{ 0.0, 0.0, 0.0, 1.0 }
	};
	static XpexMatrix matrix_b = {
		{ 1.0, 0.0, 0.0, 0.0 },
		{ 0.0, 1.0, 0.0, 0.0 },
		{ 0.0, 0.0, 1.0, 0.0 },
		{ 0.0, 0.0, 0.0, 1.0 }
	};
	XpexMatrix composed_matrix;

	XpexPostStructure(dpy, wks1, structure1, 1.0);
	XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeEach);
	XpexExecuteDeferredActions(dpy, wks1);
	XpexRedrawAllStructures(dpy, wks1); 
	XpexUpdateWorkstation(dpy, wks1);

	XpexSetEditingMode(dpy, structure1, XpexStructure_Replace);

	XpexStoreOutputCmds(dpy, structure1);
	for (i = 0.; i < 360.; i += 1.) {
		XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeNone);
		SetElementPointerToBeginning(dpy,structure1);
		XpexSetElementPointerAtLabel(dpy, structure1, 1, 1);
		XpexRotateX(DEG_TO_RAD(i*7), matrix_a); 
		XpexRotateZ(DEG_TO_RAD(i*5), matrix_b); 
		XpexComposeMatrix(matrix_a, matrix_b, composed_matrix);
			XpexSetLocalTransform(dpy, Xpex_Replace, composed_matrix);

		XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeEach);
	}

	XpexExecuteDeferredActions(dpy, wks1);
	XpexRedrawAllStructures(dpy, wks1);


}

int
main(argc, argv)
	int argc;
	char *argv[];
{
	Initialize();
	for(;;)
		DrawImage();
}
