/* $Header: XpexColorAttrs.c,v 2.1 91/07/11 12:34:51 sinyaw Exp $ */
/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

#include "Xpexlibint.h"
#include "Xpexlib.h"
#include <X11/extensions/Xext.h>
#include "extutil.h"
#include "Xpextutil.h"

void
XpexSetLightState(dpy, enable, numEnable, disable, numDisable)
	Display *dpy;
	XpexTableIndex *enable;
	int numEnable;
	XpexTableIndex *disable;
	int numDisable;
{
	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);
	XpexCheckLastReq(dpy,i);

	LockDisplay(dpy);	
	{
		char *pStream;
		pexLightState *oc;
		pexTableIndex *e, *d;
		int a = numEnable * sizeof(pexTableIndex);
		int b = numDisable * sizeof(pexTableIndex);
		int extra = a + PADDING(a) + b + PADDING(b);
		XpexGetOutputCmdExtra(LightState, dpy, extra, oc);
		oc->numEnable = (CARD16) numEnable;
		oc->numDisable = (CARD16) numDisable;

		pStream = (char *)(oc + 1);
		bcopy((char *)enable,pStream,a + PADDING(a));
		pStream += a + PADDING(a);
		bcopy((char *)disable,pStream,a + PADDING(a));
	}
	UnlockDisplay(dpy);	
}

void
XpexSetDepthCueIndex(dpy, index)
	Display *dpy;
	XpexTableIndex index;
{
	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);
	XpexCheckLastReq(dpy,i);

	LockDisplay(dpy);
	{
		pexDepthCueIndex *oc;
		XpexGetOutputCmd(DepthCueIndex, dpy, oc);
		oc->index = (pexTableIndex) index;
	}
	UnlockDisplay(dpy);
}

void
XpexSetColorApproxIndex(dpy, index)
	Display *dpy;
	XpexTableIndex index;
{
	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);
	XpexCheckLastReq(dpy,i);

	LockDisplay(dpy);
	{
		pexColourApproxIndex *oc;
		XpexGetOutputCmd(ColourApproxIndex, dpy, oc);
		oc->index = (pexTableIndex) index;
	}
	UnlockDisplay(dpy);
}

void
XpexSetRenderingColorModel(dpy, model)
	Display *dpy;
	XpexEnumTypeIndex model;
{
	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);
	XpexCheckLastReq(dpy,i);

	LockDisplay(dpy);	
	{
		pexRenderingColourModel *oc;
		XpexGetOutputCmd(RenderingColourModel, dpy, oc);
		oc->model = (pexEnumTypeIndex) model;
	}
	UnlockDisplay(dpy);	
}

void
XpexSetParaSurfChar(dpy, psc)
	Display *dpy;
	XpexParaSurfChar *psc;
{
	static void pack_psc_data();

	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);
	XpexCheckLastReq(dpy,i);

	LockDisplay(dpy);	
	{
		extern int size_of_psc_data();

		pexParaSurfCharacteristics *oc;
		int bytesWritten;
		int extra = size_of_psc_data(psc);
		XpexGetOutputCmdExtra(ParaSurfCharacteristics, dpy, extra, oc);
		oc->characteristics = (pexEnumTypeIndex) psc->characteristics;
		oc->length = (CARD16) extra;
		pack_psc_data(psc,(char *)(oc +1), &bytesWritten);
	}
	UnlockDisplay(dpy);	
}

static void
pack_psc_data(src, dest, length)
	XpexParaSurfChar *src;
	char *dest; /* RETURN */
	int *length; /* RETURN */
{
	*length = 0;

	switch (src->characteristics) {
	case PEXPSCIsoCurves:
	{
		pexPSC_IsoparametricCurves *psc =
		(pexPSC_IsoparametricCurves *) dest;
		psc->placementType = src->record.iso_curves.placementType;
		psc->numUcurves = src->record.iso_curves.numUcurves;
		psc->numVcurves = src->record.iso_curves.numVcurves;
		*length = sizeof(pexPSC_IsoparametricCurves);
		break;
	}
	case PEXPSCMcLevelCurves:
	{
		register int count;
		XpexCoord3D *s;
		pexPSC_LevelCurves *psc = (pexPSC_LevelCurves *) dest;
		psc->origin.x = (FLOAT) src->record.mc_curves.origin.x;
		psc->origin.y = (FLOAT) src->record.mc_curves.origin.y;
		psc->origin.z = (FLOAT) src->record.mc_curves.origin.z;
		psc->direction.x = (FLOAT) src->record.mc_curves.direction.x;
		psc->direction.y = (FLOAT) src->record.mc_curves.direction.y;
		psc->direction.z = (FLOAT) src->record.mc_curves.direction.z;
		count = src->record.mc_curves.numberIntersections;
		psc->numberIntersections = (CARD16) count;
		*length = sizeof(pexPSC_LevelCurves) +
		(count * sizeof(pexCoord3D));
		dest += sizeof(pexPSC_LevelCurves);
		s = src->record.mc_curves.intersections;
		bcopy((char *)s,(char *)dest,count * sizeof(pexCoord3D));
		break;
	}
	case PEXPSCWcLevelCurves:
	{
		register int count;
		XpexCoord3D *s;
		pexPSC_LevelCurves *psc = (pexPSC_LevelCurves *) dest;
		psc->origin.x = (FLOAT) src->record.wc_curves.origin.x;
		psc->origin.y = (FLOAT) src->record.wc_curves.origin.y;
		psc->origin.z = (FLOAT) src->record.wc_curves.origin.z;
		psc->direction.x = (FLOAT) src->record.wc_curves.direction.x;
		psc->direction.y = (FLOAT) src->record.wc_curves.direction.y;
		psc->direction.z = (FLOAT) src->record.wc_curves.direction.z;
		count = src->record.wc_curves.numberIntersections;
		psc->numberIntersections = (CARD16) count;
		*length = sizeof(pexPSC_LevelCurves) +
		(count * sizeof(pexCoord3D));
		dest += sizeof(pexPSC_LevelCurves);
		s = src->record.wc_curves.intersections;
		bcopy((char *)s,(char *)dest,count * sizeof(pexCoord3D));
		break;
	}
	case PEXPSCNone :
	case PEXPSCImpDep:
	default:
	 	*length = 0;
		break;
	} /* end-switch */
}
