/* $Header: pexrdrinfo.c,v 2.1 91/07/11 12:36:50 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/* pexrdrinfo */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "Xpexlibint.h"
#include "Xpexlib.h"
#include "Xpexutil.h"

#define BORDER_WIDTH   2

static Window openWindow();

static char *window_name = "";

static char *displayName = NULL;
static Display *dpy;
static Window drawable;

static XpexRenderer rdr_id;

main(argc, argv)
	int argc;
	char *argv[];
{
	InitX();
	InitPEX();

	dpy_rdr_info();  
	clean_up();
}

InitX()
{
	dpy = XOpenDisplay(displayName);

	if (!dpy) {
		exit(-1);
	}

	XSynchronize(dpy, 1);

	drawable = openWindow(50,50, 100, 100, 1); 
}

InitPEX()
{
	XExtCodes *pex_codes;
	XpexExtensionInfo info;

	pex_codes = XpexInitialize(dpy, &info);

	if (!pex_codes) {
		exit(-1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

static Window
openWindow(x, y, width, height, flag)
	int x, y;
	int width, height;
	int flag;
{
	XSetWindowAttributes theWindowAttributes;
	XSizeHints theSizeHints;
	unsigned long theWindowMask;
	Window theNewWindow;
	int theScreen = DefaultScreen(dpy);
	int theDepth = DefaultDepth(dpy, theScreen);

	theWindowAttributes.border_pixel = BlackPixel(dpy, theScreen);
	theWindowAttributes.background_pixel = WhitePixel(dpy, theScreen);
	theWindowAttributes.override_redirect = True;
	theWindowMask = CWBackPixel | CWBorderPixel | CWOverrideRedirect;

	theNewWindow = XCreateWindow(dpy,
		RootWindow(dpy, theScreen),
		x, y,
		width, height,
		BORDER_WIDTH,
		theDepth,
		InputOutput,
		CopyFromParent,
		theWindowMask,
		&theWindowAttributes);

	theSizeHints.flags = PPosition | PSize;
	theSizeHints.x = x;
	theSizeHints.y = y;
	theSizeHints.width = width;
	theSizeHints.height = height;

	XSetNormalHints(dpy, theNewWindow, &theSizeHints);

	XMapWindow(dpy, theNewWindow);

	XFlush(dpy);

	return(theNewWindow);
}

dpy_rdr_info()
{
	static char *rdr_msg_str[24] = {
		"Renderer Attributes (Default Values)\n\n",
		"\tPipelineContext   : %u\n",
		"\tCurrentPath       : %d\n",
		"\tMarkerBundle      : %u\n",
		"\tTextBundle        : %u\n",
		"\tLineBundle        : %u\n",
		"\tInteriorBundle    : %u\n",
		"\tEdgeBundle        : %u\n",
		"\tViewTable         : %u\n",
		"\tColourTable       : %u\n",
		"\tDepthCueTable     : %u\n",
		"\tLightTable        : %u\n",
		"\tColourApproxTable : %u\n",
		"\tPatternTable      : %u\n",
		"\tTextFontTable     : %u\n",
		"\tHighLightIncl     : %u\n",
		"\tHighlightExcl     : %u\n",
		"\tInvisibilityIncl  : %u\n",
		"\tInvisibilityExcl  : %u\n",
		"\tRendererState     : %s(%d)\n",
		"\tHlhsrMode         : %d\n",
		"\tNpcSubvolume      : {(%.1f,%.1f,%.1f),(%.1f,%.1f,%.1f)}\n",
		"\tViewport          : {(%d,%d,%.1f),(%d,%d,%.1f),%u}\n",
		"\tClipList          : %d\n"
	};
	XpexRendererAttributes values;
	XpexBitmask valueMask;

	rdr_id = XpexCreateRenderer(dpy, drawable, 
	(XpexBitmask) 0, (XpexRendererAttributes *) NULL);

	(void) fprintf(stderr, rdr_msg_str[0]);

	valueMask = XpexRD_PipelineContext; 

	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[1], values.pc_id);

	valueMask = XpexRD_CurrentPath; 
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[2], values.current_path.num_refs); 

	valueMask = XpexRD_MarkerBundle;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[3], values.marker_bundle);

	valueMask = XpexRD_TextBundle;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[4], values.text_bundle);

	valueMask = XpexRD_LineBundle;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[5], values.line_bundle);

	valueMask = XpexRD_InteriorBundle;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[6], values.interior_bundle);

	valueMask = XpexRD_EdgeBundle |
		XpexRD_ViewTable |
		XpexRD_ColorTable ;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[7], values.edge_bundle);
	(void) fprintf(stderr, rdr_msg_str[8], values.view_table);
	(void) fprintf(stderr, rdr_msg_str[9], values.color_table);

	valueMask = XpexRD_DepthCueTable;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[10], values.depth_cue_table);

	valueMask = XpexRD_LightTable;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[11], values.light_table);

	valueMask = XpexRD_ColorApproxTable;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[12], values.color_approx_table);

	valueMask = XpexRD_PatternTable;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[13], values.pattern_table);

	valueMask = XpexRD_TextFontTable;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[14], values.text_font_table);

	valueMask = XpexRD_HighlightIncl |
		XpexRD_HighlightExcl |
		XpexRD_InvisibilityIncl;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[15], values.highlight_incl);
	(void) fprintf(stderr, rdr_msg_str[16], values.highlight_excl);
	(void) fprintf(stderr, rdr_msg_str[17], values.invis_incl);

	valueMask = XpexRD_InvisibilityExcl;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[18], values.invis_excl);

	valueMask = XpexRD_RendererState;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[19], 
	((values.renderer_state) ? "Rendering" : "Idle"), values.renderer_state);

	valueMask = XpexRD_HlhsrMode |
		XpexRD_NpcSubvolume |
		XpexRD_Viewport;
	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);

	(void) fprintf(stderr, rdr_msg_str[20], values.hlhsr_mode);

	(void) fprintf(stderr, rdr_msg_str[21], 
		values.npc_subvolume.minval.x, values.npc_subvolume.minval.y,
		values.npc_subvolume.minval.z,
		values.npc_subvolume.maxval.x, values.npc_subvolume.maxval.y,
		values.npc_subvolume.maxval.z);
	(void) fprintf(stderr, rdr_msg_str[22], 
		values.viewport.minval.x,
		values.viewport.minval.y,
		values.viewport.minval.z,
		values.viewport.maxval.x,
		values.viewport.maxval.y,
		values.viewport.maxval.z,
		values.viewport.useDrawable); 

	valueMask = XpexRD_ClipList;

	XpexGetRendererAttributes(dpy, rdr_id, valueMask, &values);
	(void) fprintf(stderr, rdr_msg_str[23], values.clip_list.num_rects);

	(void) fprintf(stderr,"\n");

	dpy_rdr_dyn();
}

dpy_rdr_dyn()
{
	XpexRendererDynamics rdr_dynamics;

	(void) fprintf(stderr, 
	"Renderer Dynamics (Default Values)\n\n");

	XpexGetRendererDynamics(dpy, rdr_id, &rdr_dynamics);

	(void) fprintf(stderr, 
	"\tTables     : 0x%x\n", rdr_dynamics.tables);

	(void) fprintf(stderr, 
	"\tNamesets   : 0x%x\n", rdr_dynamics.namesets);

	(void) fprintf(stderr, 
	"\tAttributes : 0x%x\n", rdr_dynamics.attributes);
}

clean_up()
{
	XpexFreeRenderer(dpy, rdr_id);
}
